@@ -371,6 +371,22 @@ class HtmlParserTestCase(HelperTestCase):
              ('end', root[1][0]), ('end', root[1]), ('end', root)],
             events)
 
+    def test_html_iterparse_broken_meta(self):
+        # Broken HTML with a misplaced tag before the real html tag.
+        body = '''
+        <meta http-equiv="X-UA-Compatible" content="IE=edge" />
+        <html>
+            <head></head>
+            <body>
+            </body>
+        </html>
+        '''
+        PARSE_TAGS = {'meta', 'html', 'body'}
+
+        iterator = etree.iterparse(
+            BytesIO(body.encode()), events=('start', 'end'), html=True, recover=True, tag=PARSE_TAGS)
+        parse_events = list(iterator)
+
     def test_html_iterparse_broken_no_recover(self):
         iterparse = self.etree.iterparse
         f = BytesIO('<p>P<br></div>')
@@ -661,7 +677,6 @@ class HtmlParserTestCase(HelperTestCase):
             b'<html xmlns="http://www.w3.org/1999/xhtml"></html>'
         )
         root = html.fromstring(xhtml)
-        print(root.attrib)
         result = etree.tostring(root)
         self.assertEqual(result, b'<html xmlns="http://www.w3.org/1999/xhtml"/>')
 
@@ -673,7 +688,6 @@ class HtmlParserTestCase(HelperTestCase):
             b'<html xmlns="http://www.w3.org/1999/xhtml"></html>'
         )
         root = html.fromstring(xhtml)
-        print(root.attrib)
         result = etree.tostring(root)
         self.assertEqual(result, b'<html xmlns="http://www.w3.org/1999/xhtml"/>')
         """
@@ -681,7 +695,7 @@ class HtmlParserTestCase(HelperTestCase):
 
 def test_suite():
     suite = unittest.TestSuite()
-    suite.addTests([unittest.makeSuite(HtmlParserTestCase)])
+    suite.addTests([unittest.defaultTestLoader.loadTestsFromTestCase(HtmlParserTestCase)])
     return suite
 
 
